/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.smx.components.attachments;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.util.ArrayList;
import java.util.List;

import javax.activation.DataHandler;
import javax.activation.FileDataSource;
import javax.jbi.messaging.NormalizedMessage;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
/**
 * @author Andrea Zoppello
 */
public class AttachmentsUtils {
	
	private static final Log log = LogFactory.getLog(AttachmentsUtils.class); 
	
	public static byte[] getAttachmentAsByteArray(DataHandler dh) throws IOException {
		ByteArrayOutputStream bos = new ByteArrayOutputStream();
		writeAttachmentToOutputStrean(dh, bos);
		return bos.toByteArray();
	}
	
	public static DataHandler getFileAsAttachment(String fileName){
		return getFileAsAttachment(new File(fileName));
	}
	
	public static DataHandler getFileAsAttachment(File file){
		return new DataHandler(new FileDataSource(file));	
	}
	
	
	public static void writeAttachmentToFile(DataHandler dh, String fileName)  throws IOException {
		File destinationFile = new File(fileName);
		FileOutputStream outputStream = new FileOutputStream(destinationFile);
		writeAttachmentToOutputStrean(dh, outputStream);
		
	}
	
	public static void writeAttachmentToFile(DataHandler dh, File destinationFile)  throws IOException {
		FileOutputStream outputStream = new FileOutputStream(destinationFile);
		writeAttachmentToOutputStrean(dh, outputStream);	
	}
	
	public static void writeAttachmentToOutputStrean(DataHandler dh, OutputStream os) throws IOException {
		dh.writeTo(os);
		os.flush();
	}
	
	public static List<String> getAttachmentNames(NormalizedMessage message) throws IOException {
		List<String> attachmentNames = new ArrayList<String>();
		attachmentNames.addAll(message.getAttachmentNames());
		return attachmentNames;
	}

	
}
