/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.smx.components.attachments;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.activation.DataHandler;
import javax.jbi.messaging.Fault;
import javax.jbi.messaging.MessageExchange;
import javax.jbi.messaging.NormalizedMessage;

import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.MessageExchangeListener;
import org.apache.servicemix.jbi.FaultException;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.apache.servicemix.jbi.jaxp.StringSource;
import org.apache.servicemix.jbi.util.ByteArrayDataSource;
import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Node;
import org.dom4j.XPath;
import org.dom4j.xpath.DefaultXPath;
/**
  * @author andrea zoppello
  * The  MessageContentToAttachments takes the attachmnents normalized as base64 attachments and put
  * into message as normalized attachments
  * 
  */
public class MessageContentToAttachments extends AttachmentTransformComponentSupport implements
	MessageExchangeListener {
	
	private static final Log log = LogFactory.getLog(MessageContentToAttachments.class);
	
	
	
	
	
	@Override
	protected boolean transform(MessageExchange exchange, NormalizedMessage in, NormalizedMessage out) throws Exception {
		
		SourceTransformer sourceTransformer = new SourceTransformer();
    	
		try {
    		log.debug("MessageContentToAttachment-> start");
    		
    		log.debug("MessageContentToAttachment-> getting input message content");
    		String xmlInput= sourceTransformer.contentToString(in);
    		log.debug("MessageContentToAttachment->"+xmlInput);
    		log.debug("MessageContentToAttachment->Parsing input xml");
    		Document inputDocument = DocumentHelper.parseText(xmlInput);
    		log.debug("MessageContentToAttachment->Parsing OK");
    		
    		log.debug("MessageContentToAttachment-> Evaluating attachment nodes ["+getBase64attachmentElementName()+"]");
    		//List<Node> attachmentNodes = inputDocument.selectNodes("//"+getBase64attachmentElementName());
    		XPath xPathExpression = new DefaultXPath("//"+getBase64attachmentElementName());
			Map<String, String> namespaceURI = new HashMap<String, String>();
			namespaceURI.put("spagic", "urn:it:eng:spagic");
    		xPathExpression.setNamespaceURIs(namespaceURI);
    		List<Node> attachmentNodes = xPathExpression.selectNodes(inputDocument);
    		
    		
    		log.debug("MessageContentToAttachment-> Founded ["+attachmentNodes.size()+"] attachments nodes");
    		String name = null;
    		String contentType = null;
    		byte[] decodedBytes = null;
    		String base64Text = null;
    		for (Node n : attachmentNodes){
    			name = n.valueOf("@name");
    			contentType = n.valueOf("@content-type");
    			
    			if ( name == null){
    				log.info(" Attribute name not found setting ");
    				name = "attachment"+ System.currentTimeMillis();
    			}
    			if (contentType == null){
    				log.info(" Attribute content-type not found setting ");
    				contentType ="application/octet-stream";
    			}
    			base64Text = n.getText();
    			DataHandler dh  = new DataHandler(new ByteArrayDataSource(Base64.decodeBase64(base64Text.getBytes("UTF-8")),contentType));
    	    	out.addAttachment(name, dh);
    	    	
    		}	
    		
    		boolean removeResult = false;
    		for (Node n : attachmentNodes){
    			log.debug("Removing base64 attachment Node");
    			//inputDocument.remove(n);
    			n.detach();
    		}
    		
    		out.setContent(new StringSource(inputDocument.asXML()));
    		
    		log.debug("MessageContentToAttachment-> end");
    		
    		
    		return true;
		
		}catch(Exception e){
    		e.printStackTrace();
    		log.error("Exception AttachmentsToMessageContent");
    		log.error(e);
    		Fault fault = exchange.createFault();
        	fault.setContent(new StringSource("<ERROR>AttachmentsToMessageContent produced a null message</ERROR>"));
        	fault.setProperty("org.apache.servicemix.xml", in.getContent());
        	throw new FaultException("AttachmentsToMessageContent Failed ", exchange, fault);
    	}
	} 
	
	
	
   

}
