/**

    Copyright 2007, 2008 Engineering Ingegneria Informatica S.p.A.

    This file is part of Spagic.

    Spagic is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
    the Free Software Foundation; either version 3 of the License, or
    any later version.

    Spagic is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU Lesser General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
**/
package org.spagic.smx.components.jdbcquery;

import javax.jbi.messaging.NormalizedMessage;
import javax.xml.xpath.XPath;
import javax.xml.xpath.XPathFactory;

import org.apache.commons.beanutils.ConvertUtils;
import org.apache.commons.codec.binary.Base64;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.servicemix.jbi.jaxp.SourceTransformer;
import org.w3c.dom.Node;

/**
 * Factory that initialize query parameter based on configuration and on
 * values retrived from input message.
 * 
 * @author buso
 *
 */
public class JDBCParameterFactory {
	private static final Log log = LogFactory.getLog(JDBCParameterFactory.class);
	private static JDBCParameterFactory singleton;
	
	public static JDBCParameterFactory getInstance() {
		if (singleton == null) {
			singleton = new JDBCParameterFactory();
		}
		return singleton;
	}
	
	public Object initializeParameter(NormalizedMessage msg,
		QueryParameterConfig config) throws Exception {
		
		// TODO check to consider namespaces
		
		// TODO: remove build of the dom for each xpath mapping in the retrieveMessageInEnvelope
		Object xmlValue = retrieveMessageInEnvelope(msg,
			config.getXpath());
		
		log.debug("Converting value: "+xmlValue+" to type: "+getClassFromConfig(config));
		Object value = ConvertUtils
			.convert((String)xmlValue, getClassFromConfig(config));
		
		if ( isBase64(config) ) {
			return Base64.decodeBase64(value.toString().getBytes());
		}
		
		return value;
	}
	
    private Object retrieveMessageInEnvelope(NormalizedMessage in, String xpathString)
		throws Exception {
	
		SourceTransformer domTransformer = new SourceTransformer();
		
		Node node = domTransformer.toDOMDocument(in.getContent());
		XPath xpath = XPathFactory.newInstance().newXPath(); 
		String nmEnvelope = xpath.evaluate(xpathString, node);
		if (nmEnvelope == null) {
			throw new Exception("Normalize message envelope ("+xpathString+") not found in Input NM");
		}
		return nmEnvelope;
	}

	private boolean isBase64(QueryParameterConfig config) {
		return QueryParameterConfig.BINARY_BASE64_TYPE
			.equals(config.getParamType());
	}
	
	private Class getClassFromConfig(QueryParameterConfig config)
		throws ClassNotFoundException {
		
		if ( isBase64(config) ) {
			
			return String.class;
		}
		return Class.forName(config.getParamType());
	}
}
